import sys
import argparse
import importlib
import traceback


def get_available_modules():
    """Return the list of available modules for custom_exporter."""
    available_modules = [
        # Disk modules
        "disk.xfs",
        "disk.podman",
        
        # Mail queue modules
        "mailq.generic",
        "mailq.podman",
        
        # Backup modules
        "backups.users",
        "backups.vms",
        
        # Service modules
        "service.podman",
        
        # DP-HA-CTL modules
        "dphactl.core",
        "dphactl.systemctl",
        "dphactl.logging_checks",
        "dphactl.containers",
        "dphactl.verify",
    ]
    
    return available_modules


def display_help():
    """Display help text with a list of available modules."""
    modules = get_available_modules()
    
    help_text = """
USAGE: custom_exporter <module_name> [arguments]

Available modules:
"""
    
    for module in modules:
        help_text += f"  - {module}\n"
    
    help_text += """
Examples:
  custom_exporter disk.xfs
  custom_exporter mailq.generic
  custom_exporter dphactl.core
  
For detailed documentation, see the README.md file.
"""
    print(help_text)


def main():
    parser = argparse.ArgumentParser(
        prog='custom_exporter',
        description='Custom exporters for Prometheus monitoring',
        add_help=False  # We'll handle --help ourselves for more custom output
    )
    parser.add_argument('command', nargs='?', default=None, 
                        help='The module to run (e.g., disk.xfs, mailq.generic)')
    parser.add_argument('command_args', nargs='*', default=[], type=str,
                        help='Arguments to pass to the module')
    parser.add_argument('-h', '--help', action='store_true', 
                        help='Show this help message')
    
    args = parser.parse_args()
    
    # Show help if requested or if no command was provided
    if args.help or args.command is None:
        display_help()
        sys.exit(0)
    
    module_name = "custom_exporters.%s" % args.command

    try:
        imported_module = importlib.import_module(module_name)
    except ModuleNotFoundError:
        print(f"Error: Module '{args.command}' not found")
        print("\nAvailable modules:")
        for module in get_available_modules():
            print(f"  - {module}")
        print("\nFor more information, run: custom_exporter --help")
        sys.exit(1)
    except Exception:
        traceback.print_exc(file=sys.stdout)
        print(f"Failed to load module {module_name}")
        sys.exit(1)

    try:
        cls = imported_module.CustomExporter(args=args.command_args)
    except Exception:
        traceback.print_exc(file=sys.stdout)
        print("failed to load CustomExporter class from module %s" % module_name)
        sys.exit(1)

    try:
        cls.run()
    except Exception as e:
        traceback.print_exc(file=sys.stdout)
        print("generate failed: %s" % e)
        sys.exit(1)

    print(cls.metrics.dump())
